#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <Evas.h>

#include "ephysics_private.h"

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct _EPhysics_Body_Callback EPhysics_Body_Callback;

struct _EPhysics_Body_Callback {
     EINA_INLIST;
     void (*func) (void *data, EPhysics_Body *body, void *event_info);
     void *data;
     EPhysics_Callback_Body_Type type;
};

struct _EPhysics_Body {
     btCollisionShape *collision_shape;
     btRigidBody *rigid_body;
     Evas_Object *evas_obj;
     EPhysics_World *world;
     void *data;
     Eina_Inlist *callbacks;
     double mass;
     Eina_Bool active:1;
     Eina_List *collision_groups;
};

struct _EPhysics_Body_Collision {
     EPhysics_Body *contact_body;
     Evas_Coord x;
     Evas_Coord y;
};

void
ephysics_body_active_set(EPhysics_Body *body, Eina_Bool active)
{
   EPhysics_Body_Callback *cb;

   if (body->active == !!active) return;
   body->active = !!active;
   if (active) return;

   EINA_INLIST_FOREACH(body->callbacks, cb)
      if (cb->type == EPHYSICS_CALLBACK_BODY_STOPPED)
        cb->func(cb->data, body, (void *) body->evas_obj);
};

Eina_Bool
ephysics_body_filter_collision(EPhysics_Body *body0, EPhysics_Body *body1)
{
   void *grp;
   Eina_Iterator *it;
   Eina_List *l;

   if ((!body0->collision_groups) || (!body1->collision_groups))
     return EINA_TRUE;

   EINA_LIST_FOREACH(body0->collision_groups, l, grp)
     {
        if (eina_list_data_find(body1->collision_groups, grp))
          return EINA_TRUE;
     }

   return EINA_FALSE;
}

EAPI Eina_Bool
ephysics_body_collision_group_add(EPhysics_Body *body, const char *group)
{
   Eina_Stringshare *group_str;

   if (!body)
     {
        ERR("Can't add body collision group, body is null.");
        return EINA_FALSE;
     }

   group_str = eina_stringshare_add(group);
   if (eina_list_data_find(body->collision_groups, group_str))
     {
        INF("Body already added to group: %s", group);
        eina_stringshare_del(group_str);
        return EINA_TRUE;
     }

   body->collision_groups = eina_list_append(body->collision_groups, group_str);
   return EINA_TRUE;
}

EAPI Eina_Bool
ephysics_body_collision_group_del(EPhysics_Body *body, const char *group)
{
   Eina_Stringshare *group_str;

   if (!body)
     {
        ERR("Can't remove body collision group, body is null.");
        return EINA_FALSE;
     }

   group_str = eina_stringshare_add(group);
   if (!eina_list_data_find(body->collision_groups, group_str))
     {
        INF("Body isn't part of group: %s", group);
        eina_stringshare_del(group_str);
        return EINA_TRUE;
     }

   body->collision_groups = eina_list_remove(body->collision_groups, group_str);
   eina_stringshare_del(group_str);
   eina_stringshare_del(group_str);
   return EINA_TRUE;
}

EAPI const Eina_List *
ephysics_body_collision_group_list_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get the body's collision group, body is null.");
        return NULL;
     }

   return body->collision_groups;
}

static EPhysics_Body *
_ephysics_body_add(EPhysics_World *world, btCollisionShape *collision_shape)
{
   EPhysics_Body *body;
   btScalar mass = 1;

   body = (EPhysics_Body *) calloc(1, sizeof(EPhysics_Body));
   if (!body)
     {
        ERR("Couldn't create a new body instance.");
        return NULL;
     }

   btDefaultMotionState *motion_state =
      new btDefaultMotionState();
   if (!motion_state)
     {
        ERR("Couldn't create a motion state.");
        free(body);
        return NULL;
     }

   btVector3 inertia(0, 0, 0);
   collision_shape->calculateLocalInertia(mass, inertia);

   btRigidBody::btRigidBodyConstructionInfo rigid_body_ci(
      mass, motion_state, collision_shape, inertia);
   btRigidBody *rigid_body = new btRigidBody(rigid_body_ci);
   if (!rigid_body)
     {
        ERR("Couldn't create a rigid body.");
        delete motion_state;
        free(body);
        return NULL;
     }

   body->collision_groups = NULL;
   body->collision_shape = collision_shape;
   body->rigid_body = rigid_body;
   body->mass = mass;
   body->world = world;
   body->rigid_body->setUserPointer(body);
   body->rigid_body->setLinearFactor(btVector3(1, 1, 0));
   body->rigid_body->setAngularFactor(btVector3(0, 0, 1));

   return body;
}

static void
_ephysics_body_evas_obj_del_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
   EPhysics_Body *body = (EPhysics_Body *) data;
   body->evas_obj = NULL;
   DBG("Evas object deleted. Updating body: %p", body);
}

static void
_ephysics_body_del(EPhysics_Body *body)
{
   EPhysics_Body_Callback *cb;
   void *group;

   if (body->evas_obj)
     evas_object_event_callback_del(body->evas_obj, EVAS_CALLBACK_DEL,
                                    _ephysics_body_evas_obj_del_cb);

   while (body->callbacks)
     {
        cb = EINA_INLIST_CONTAINER_GET(body->callbacks,
                                         EPhysics_Body_Callback);
        body->callbacks = eina_inlist_remove(body->callbacks, body->callbacks);
        free(cb);
     }

   EINA_LIST_FREE(body->collision_groups, group)
        eina_stringshare_del((Eina_Stringshare *)group);

   delete body->rigid_body->getMotionState();
   delete body->collision_shape;
   delete body->rigid_body;

   free(body);
}

static void
_ephysics_body_geometry_set(EPhysics_Body *body, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
{
   double rate, mx, my, sx, sy;
   btTransform trans;
   int wy, height;

   rate = ephysics_world_rate_get(body->world);
   ephysics_world_render_geometry_get(body->world, NULL, &wy, NULL, &height);
   height += wy;

   mx = (x + w / 2) / rate;
   my = (height - (y + h / 2)) / rate;
   sx = w / rate;
   sy = h / rate;

   body->rigid_body->getMotionState()->getWorldTransform(trans);
   trans.setOrigin(btVector3(mx, my, 0));
   body->rigid_body->proceedToTransform(trans);

   body->collision_shape->setLocalScaling(btVector3(sx, sy, 1));

   if(!body->rigid_body->isStaticObject())
      ephysics_body_mass_set(body, ephysics_body_mass_get(body));

   body->rigid_body->getMotionState()->setWorldTransform(trans);

   DBG("Body %p position changed to %lf, %lf.", body, mx, my);
   DBG("Body %p scale changed to %lf, %lf.", body, sx, sy);
}

static void
_ephysics_body_evas_object_default_update(EPhysics_Body *body)
{
   int x, y, w, h, wx, wy, wh, cx, cy;
   EPhysics_Camera *camera;
   btTransform trans;
   double rate, rot;
   Evas_Map *map;

   if (!body->evas_obj)
     return;

   body->rigid_body->getMotionState()->getWorldTransform(trans);
   ephysics_world_render_geometry_get(body->world, &wx, &wy, NULL, &wh);

   camera = ephysics_world_camera_get(body->world);
   ephysics_camera_position_get(camera, &cx, &cy);
   cx -= wx;
   cy -= wy;

   evas_object_geometry_get(body->evas_obj, NULL, NULL, &w, &h);
   rate = ephysics_world_rate_get(body->world);
   x = (int) (trans.getOrigin().getX() * rate) - w / 2 - cx;
   y = wh + wy - (int) (trans.getOrigin().getY() * rate) - h / 2 - cy;

   evas_object_move(body->evas_obj, x, y);

   rot = - trans.getRotation().getAngle() * RAD_TO_DEG *
      trans.getRotation().getAxis().getZ();

   map = evas_map_new(4);
   evas_map_util_points_populate_from_object(map, body->evas_obj);
   evas_map_util_rotate(map, rot, x + (w / 2), y + (h / 2));
   evas_object_map_set(body->evas_obj, map);
   evas_object_map_enable_set(body->evas_obj, EINA_TRUE);
   evas_map_free(map);
}

static void
_ephysics_body_outside_render_area_check(EPhysics_Body *body)
{
   int wx, wy, ww, wh, bx, by, bw, bh;

   ephysics_world_render_geometry_get(body->world, &wx, &wy, &ww, &wh);
   ephysics_body_geometry_get(body, &bx, &by, &bw, &bh);

   // FIXME: check what should be done regarding rotated bodies
   if (((ephysics_world_bodies_outside_top_autodel_get(body->world)) &&
        (by + bh < wy)) ||
       ((ephysics_world_bodies_outside_bottom_autodel_get(body->world)) &&
        (by > wy + wh)) ||
       ((ephysics_world_bodies_outside_left_autodel_get(body->world)) &&
        (bx + bh < wx)) ||
       ((ephysics_world_bodies_outside_right_autodel_get(body->world)) &&
        (bx > wx + ww)))
     {
        ephysics_world_body_del(body->world, body, body->rigid_body);
        ephysics_orphan_body_del(body);
        DBG("Body %p deleted. Out of render area", body);
     }
}

void
ephysics_body_evas_object_update_select(EPhysics_Body *body)
{
   Eina_Bool callback_called = EINA_FALSE;
   EPhysics_Body_Callback *cb;

   if (!body)
     return;

   EINA_INLIST_FOREACH(body->callbacks, cb)
     {
        if (cb->type == EPHYSICS_CALLBACK_BODY_UPDATE) {
             cb->func(cb->data, body, (void *) body->evas_obj);
             callback_called = EINA_TRUE;
        }
     }

   if (!callback_called)
     _ephysics_body_evas_object_default_update(body);

   if (ephysics_world_bodies_outside_autodel_get(body->world))
     _ephysics_body_outside_render_area_check(body);
}

static void
_ephysics_body_collision_set(EPhysics_Body_Collision *collision, EPhysics_Body *contact_body, btVector3 position)
{
   double rate;
   int wy, wh;
   EPhysics_World *world = contact_body->world;

   ephysics_world_render_geometry_get(world, NULL, &wy, NULL, &wh);
   rate = ephysics_world_rate_get(world);
   collision->contact_body = contact_body;
   collision->x = position.getX() * rate;
   collision->y = wh + wy - (position.getY() * rate);
}

EAPI void
ephysics_body_collision_position_get(const EPhysics_Body_Collision *collision, Evas_Coord *x, Evas_Coord *y)
{
   if (!collision)
     {
        ERR("Can't get body's collision data, collision is null.");
        return;
     }

   if (x) *x = collision->x;
   if (y) *y = collision->y;
}

EAPI EPhysics_Body *
ephysics_body_collision_contact_body_get(const EPhysics_Body_Collision *collision)
{
   if (!collision)
     {
        ERR("Can't get body's collision contact body, collision is null.");
        return NULL;
     }

   return collision->contact_body;
}

void
ephysics_body_contact_processed(EPhysics_Body *body, EPhysics_Body *contact_body, btVector3 position)
{
   EPhysics_Body_Callback *cb;

   if ((!body) || (!contact_body))
     return;

   EINA_INLIST_FOREACH(body->callbacks, cb)
     {
        if (cb->type == EPHYSICS_CALLBACK_BODY_COLLISION)
          {
             EPhysics_Body_Collision *collision;

             collision = (EPhysics_Body_Collision *)malloc(
                sizeof(EPhysics_Body_Collision));

             if (!collision)
               {
                  ERR("Can't allocate collision data structure.");
                  continue;
               }

             _ephysics_body_collision_set(collision, contact_body, position);
             cb->func(cb->data, body, collision);
             free(collision);
          }
     }
}

btRigidBody *
ephysics_body_rigid_body_get(const EPhysics_Body *body)
{
   return body->rigid_body;
}

EAPI EPhysics_Body *
ephysics_body_circle_add(EPhysics_World *world)
{
   EPhysics_Body *body;

   btCollisionShape *collision_shape = new btCylinderShapeZ(
      btVector3(0.5, 0.5, 0.5));
   if (!collision_shape)
     {
        ERR("Couldn't create a cylinder shape on z.");
        return NULL;
     }

   body = _ephysics_body_add(world, collision_shape);
   if (!body)
     {
        ERR("Couldn't create a circle body.");
        delete collision_shape;
        return NULL;
     }

   if (!ephysics_world_body_add(body->world, body, body->rigid_body))
     {
        ERR("Couldn't add body to world's bodies list");
        _ephysics_body_del(body);
        return NULL;
     }

   INF("Circle body added: %p.", body);
   return body;
}

EAPI EPhysics_Body *
ephysics_body_box_add(EPhysics_World *world)
{
   EPhysics_Body *body;

   btCollisionShape *collision_shape = new btBoxShape(
      btVector3(0.5, 0.5, 0.5));
   if (!collision_shape)
     {
        ERR("Couldn't create a 2d box shape.");
        return NULL;
     }

   body = _ephysics_body_add(world, collision_shape);
   if (!body)
     {
        ERR("Couldn't create a box body.");
        delete collision_shape;
        return NULL;
     }

   if (!ephysics_world_body_add(body->world, body, body->rigid_body))
     {
        ERR("Couldn't add body to world's bodies list");
        _ephysics_body_del(body);
        return NULL;
     }

   INF("Box body added: %p.", body);
   return body;
}

void
ephysics_body_world_boundaries_resize(EPhysics_World *world)
{
   Evas_Coord x, y, width, height;
   EPhysics_Body *bottom, *top, *left, *right;

   ephysics_world_render_geometry_get(world, &x, &y, &width, &height);

   bottom = ephysics_world_boundary_get(world, EPHYSICS_WORLD_BOUNDARY_BOTTOM);
   if (bottom)
     ephysics_body_geometry_set(bottom, x, y + height, width, 10);

   right = ephysics_world_boundary_get(world, EPHYSICS_WORLD_BOUNDARY_RIGHT);
   if (right)
     ephysics_body_geometry_set(right, x + width, 0, 10, y + height);

   left = ephysics_world_boundary_get(world, EPHYSICS_WORLD_BOUNDARY_LEFT);
   if (left)
     ephysics_body_geometry_set(left,  x - 10, 0, 10, y + height);

   top = ephysics_world_boundary_get(world, EPHYSICS_WORLD_BOUNDARY_TOP);
   if (top)
     ephysics_body_geometry_set(top, 0, y - 10, x + width, 10);
}

static EPhysics_Body *
_ephysics_body_boundary_add(EPhysics_World *world, EPhysics_World_Boundary boundary, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
{
   EPhysics_Body *body;

   if (!world)
     {
        ERR("Can't add boundary, world is null.");
        return NULL;
     }

   body = ephysics_world_boundary_get(world, boundary);
   if (body)
     return body;

   body = ephysics_body_box_add(world);
   if (!body)
     return NULL;

   ephysics_body_mass_set(body, 0);
   ephysics_world_boundary_set(world, boundary, body);
   ephysics_body_geometry_set(body, x, y, w, h);

   return body;
}

EAPI EPhysics_Body *
ephysics_body_top_boundary_add(EPhysics_World *world)
{
   Evas_Coord x, y, w;
   ephysics_world_render_geometry_get(world, &x, &y, &w, NULL);
   return _ephysics_body_boundary_add(world, EPHYSICS_WORLD_BOUNDARY_TOP,
                                      0, y - 10, x + w, 10);
}

EAPI EPhysics_Body *
ephysics_body_bottom_boundary_add(EPhysics_World *world)
{
   Evas_Coord x, y, w, h;
   ephysics_world_render_geometry_get(world, &x, &y, &w, &h);
   return _ephysics_body_boundary_add(world, EPHYSICS_WORLD_BOUNDARY_BOTTOM,
                                      x, y + h, w, 10);
}

EAPI EPhysics_Body *
ephysics_body_left_boundary_add(EPhysics_World *world)
{
   Evas_Coord x, y, h;
   ephysics_world_render_geometry_get(world, &x, &y, NULL, &h);
   return _ephysics_body_boundary_add(world, EPHYSICS_WORLD_BOUNDARY_LEFT,
                                      x - 10, 0, 10, y + h);
}

EAPI EPhysics_Body *
ephysics_body_right_boundary_add(EPhysics_World *world)
{
   Evas_Coord x, y, w, h;
   ephysics_world_render_geometry_get(world, &x, &y, &w, &h);
   return _ephysics_body_boundary_add(world, EPHYSICS_WORLD_BOUNDARY_RIGHT,
                                      x + w, 0, 10, y + h);
}

void
ephysics_orphan_body_del(EPhysics_Body *body)
{
   EPhysics_Body_Callback *cb;

   EINA_INLIST_FOREACH(body->callbacks, cb)
     {
        if (cb->type == EPHYSICS_CALLBACK_BODY_DEL)
          cb->func(cb->data, body, NULL);
     }

   _ephysics_body_del(body);
}

EAPI void
ephysics_body_del(EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't delete body, it wasn't provided.");
        return;
     }

   ephysics_world_body_del(body->world, body, body->rigid_body);
   ephysics_orphan_body_del(body);

   INF("Body %p deleted.", body);
}

EAPI void
ephysics_body_evas_object_set(EPhysics_Body *body, Evas_Object *evas_obj, Eina_Bool use_obj_pos)
{
   int obj_x, obj_y, obj_w, obj_h;

   if (!body)
     {
        ERR("Can't set evas object to body, the last wasn't provided.");
        return;
     }

   if (!evas_obj)
     {
        ERR("Can't set evas object to body, the first wasn't provided.");
        return;
     }

   if (body->evas_obj)
     evas_object_event_callback_del(body->evas_obj, EVAS_CALLBACK_DEL,
                                    _ephysics_body_evas_obj_del_cb);

   body->evas_obj = evas_obj;
   evas_object_event_callback_add(evas_obj, EVAS_CALLBACK_DEL,
                                  _ephysics_body_evas_obj_del_cb, body);

   if (!use_obj_pos)
     return;

   evas_object_geometry_get(body->evas_obj, &obj_x, &obj_y, &obj_w, &obj_h);
   _ephysics_body_geometry_set(body, obj_x, obj_y, obj_w, obj_h);
}

EAPI Evas_Object *
ephysics_body_evas_object_unset(EPhysics_Body *body)
{
   Evas_Object *obj;
   if (!body)
     {
        ERR("Can't unset evas object from body, it wasn't provided.");
        return NULL;
     }

   obj = body->evas_obj;
   body->evas_obj = NULL;

   if (obj)
     evas_object_event_callback_del(obj, EVAS_CALLBACK_DEL,
                                    _ephysics_body_evas_obj_del_cb);

   return obj;
}

EAPI Evas_Object *
ephysics_body_evas_object_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get evas object from body, it wasn't provided.");
        return NULL;
     }

   return body->evas_obj;
}

EAPI void
ephysics_body_geometry_set(EPhysics_Body *body, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
{
   if (!body)
     {
        ERR("Can't set body position, body is null.");
        return;
     }

   if ((w <= 0) || (h <= 0))
     {
        ERR("Width and height must to be a non-null, positive value.");
        return;
     }

   _ephysics_body_geometry_set(body, x, y, w, h);
}

EAPI void
ephysics_body_geometry_get(const EPhysics_Body *body, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
{
   btTransform trans;
   btVector3 vector;
   double rate;
   int wy, height;

   if (!body)
     {
        ERR("Can't get body position, body is null.");
        return;
     }

   body->rigid_body->getMotionState()->getWorldTransform(trans);
   vector = body->collision_shape->getLocalScaling();

   rate = ephysics_world_rate_get(body->world);
   ephysics_world_render_geometry_get(body->world, NULL, &wy, NULL, &height);
   height += wy;

   if (x) *x = round((trans.getOrigin().getX() - vector.x() / 2) * rate);
   if (y) *y = height - round((trans.getOrigin().getY() + vector.y() / 2)
                              * rate);
   if (w) *w = round(vector.x() * rate);
   if (h) *h = round(vector.y() * rate);
}

EAPI void
ephysics_body_mass_set(EPhysics_Body *body, double mass)
{
   if (!body)
     {
        ERR("Can't set body mass, body is null.");
        return;
     }

   btVector3 inertia(0, 0, 0);
   body->collision_shape->calculateLocalInertia(mass, inertia);
   body->rigid_body->setMassProps(mass, inertia);
   body->rigid_body->updateInertiaTensor();
   body->mass = mass;

   DBG("Body %p mass changed to %lf.", body, mass);
}

EAPI double
ephysics_body_mass_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get body mass, body is null.");
        return 0;
     }

   return body->mass;
}

EAPI void
ephysics_body_linear_velocity_set(EPhysics_Body *body, double x, double y)
{
   double rate;

   if (!body)
     {
        ERR("Can't set body linear velocity, body is null.");
        return;
     }

   rate = ephysics_world_rate_get(body->world);
   body->rigid_body->setLinearVelocity(btVector3(x / rate, y / rate, 0));
   DBG("Linear velocity of body %p set to %lf, %lf", body, x, y);
}

EAPI void
ephysics_body_linear_velocity_get(const EPhysics_Body *body, double *x, double *y)
{
   double rate;

   if (!body)
     {
        ERR("Can't get linear velocity, body is null.");
        return;
     }

   rate = ephysics_world_rate_get(body->world);
   if (x) *x = body->rigid_body->getLinearVelocity().getX() * rate;
   if (y) *y = -body->rigid_body->getLinearVelocity().getY() * rate;
}

EAPI void
ephysics_body_angular_velocity_set(EPhysics_Body *body, double z)
{
   if (!body)
     {
        ERR("Can't set angular velocity, body is null.");
        return;
     }

   body->rigid_body->setAngularVelocity(btVector3(0, 0, -z/RAD_TO_DEG));
   DBG("Angular velocity of body %p set to %lf", body, z);
}

EAPI double
ephysics_body_angular_velocity_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get angular velocity, body is null.");
        return 0;
     }

   return -body->rigid_body->getAngularVelocity().getZ() * RAD_TO_DEG;
}

EAPI void
ephysics_body_sleeping_threshold_set(EPhysics_Body *body, double linear_threshold, double angular_threshold)
{
   double rate;

   if (!body)
     {
        ERR("Can't set sleeping thresholds, body is null.");
        return;
     }

   rate = ephysics_world_rate_get(body->world);
   body->rigid_body->setSleepingThresholds(linear_threshold / rate,
                                           angular_threshold / RAD_TO_DEG);
}

EAPI double
ephysics_body_linear_sleeping_threshold_get(const EPhysics_Body *body)
{
   double rate;

   if (!body)
     {
        ERR("Can't get linear sleeping threshold, body is null.");
        return 0;
     }

   rate = ephysics_world_rate_get(body->world);
   return body->rigid_body->getLinearSleepingThreshold() * rate;
}

EAPI double
ephysics_body_angular_sleeping_threshold_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get angular sleeping threshold, body is null.");
        return 0;
     }

   return body->rigid_body->getAngularSleepingThreshold() * RAD_TO_DEG;
}

EAPI void
ephysics_body_stop(EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't stop a null body.");
        return;
     }

   body->rigid_body->setLinearVelocity(btVector3(0, 0, 0));
   body->rigid_body->setAngularVelocity(btVector3(0, 0, 0));

   DBG("Body %p stopped", body);
}

EAPI void
ephysics_body_damping_set(EPhysics_Body *body, double linear_damping, double angular_damping)
{
    if (!body)
     {
        ERR("Can't set body damping, body is null.");
        return;
     }

    body->rigid_body->setDamping(btScalar(linear_damping),
                                 btScalar(angular_damping));
}

EAPI void
ephysics_body_damping_get(const EPhysics_Body *body, double *linear_damping, double *angular_damping)
{
    if (!body)
     {
        ERR("Can't get damping, body is null.");
        return;
     }

    if (linear_damping) *linear_damping = body->rigid_body->getLinearDamping();
    if (angular_damping) *angular_damping =
      body->rigid_body->getAngularDamping();
}

EAPI void
ephysics_body_evas_object_update(EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Couldn't update a null body.");
        return;
     }

   _ephysics_body_evas_object_default_update(body);
}

EAPI void
ephysics_body_event_callback_add(EPhysics_Body *body, EPhysics_Callback_Body_Type type, EPhysics_Body_Event_Cb func, const void *data)
{
   EPhysics_Body_Callback *cb;

   if (!body)
     {
        ERR("Can't set body event callback, body is null.");
        return;
     }

   if (!func)
     {
        ERR("Can't set body event callback, function is null.");
        return;
     }

   if ((type < 0) || (type >= EPHYSICS_CALLBACK_BODY_LAST))
     {
        ERR("Can't set body event callback, callback type is wrong.");
        return;
     }

   cb = (EPhysics_Body_Callback *) malloc(sizeof(EPhysics_Body_Callback));
   if (!cb)
     {
        ERR("Can't set body event callback, can't create cb instance.");
        return;
     }

   cb->func = func;
   cb->type = type;
   cb->data = (void *)data;

   body->callbacks = eina_inlist_append(body->callbacks, EINA_INLIST_GET(cb));
}

EAPI void *
ephysics_body_event_callback_del(EPhysics_Body *body, EPhysics_Callback_Body_Type type, EPhysics_Body_Event_Cb func)
{
   EPhysics_Body_Callback *cb;
   void *cb_data;

   if (!body)
     {
        ERR("Can't delete body event callback, body is null.");
        return NULL;
     }

   EINA_INLIST_FOREACH(body->callbacks, cb)
     {
        if ((cb->type == type) && (cb->func == func)) {
             cb_data = cb->data;
             body->callbacks = eina_inlist_remove(body->callbacks,
                                                  EINA_INLIST_GET(cb));
             free(cb);
             return cb_data;
        }
     }

   return NULL;
}

EAPI void *
ephysics_body_event_callback_del_full(EPhysics_Body *body, EPhysics_Callback_Body_Type type, EPhysics_Body_Event_Cb func, void *data)
{
   EPhysics_Body_Callback *cb;
   void *cb_data;

   if (!body)
     {
        ERR("Can't delete body event callback, body is null.");
        return NULL;
     }

   EINA_INLIST_FOREACH(body->callbacks, cb)
     {
        if ((cb->type == type) && (cb->func == func) && (cb->data == data)) {
             cb_data = cb->data;
             body->callbacks = eina_inlist_remove(body->callbacks,
                                                  EINA_INLIST_GET(cb));
             free(cb);
             return cb_data;
        }
     }

   return NULL;
}

EAPI void
ephysics_body_restitution_set(EPhysics_Body *body, double restitution)
{
   if (!body)
     {
        ERR("Can't set body restitution, body is null.");
        return;
     }

   body->rigid_body->setRestitution(btScalar(restitution));
}

EAPI double
ephysics_body_restitution_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get body restitution, body is null.");
        return 0;
     }

   return body->rigid_body->getRestitution();
}

EAPI void
ephysics_body_friction_set(EPhysics_Body *body, double friction)
{
   if (!body)
     {
        ERR("Can't set body friction, body is null.");
        return;
     }

   body->rigid_body->setFriction(btScalar(friction));
}

EAPI double
ephysics_body_friction_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get body friction, body is null.");
        return 0;
     }

   return body->rigid_body->getFriction();
}

EAPI EPhysics_World *
ephysics_body_world_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get the world a null body belongs to.");
        return NULL;
     }

   return body->world;
}

EAPI void
ephysics_body_central_impulse_apply(EPhysics_Body *body, double x, double y)
{
   if (!body)
     {
        ERR("Can't apply impulse to a null body.");
        return;
     }

   body->rigid_body->activate(1);
   body->rigid_body->applyCentralImpulse(btVector3(x, y, 0));
}

EAPI void
ephysics_body_linear_movement_enable_set(EPhysics_Body *body, Eina_Bool enable_x, Eina_Bool enable_y)
{
   if (!body)
     {
        ERR("Can't set linear factor on a null body.");
        return;
     }

   body->rigid_body->setLinearFactor(btVector3(!!enable_x, !!enable_y, 0));
}

EAPI void
ephysics_body_linear_movement_enable_get(const EPhysics_Body *body, Eina_Bool *enable_x, Eina_Bool *enable_y)
{
   if (!body)
     {
        ERR("Can't check if linear factor is enabled, body is null.");
        return;
     }

   if (enable_x) *enable_x = !!body->rigid_body->getLinearFactor().x();
   if (enable_y) *enable_y = !!body->rigid_body->getLinearFactor().y();
}

EAPI void
ephysics_body_torque_impulse_apply(EPhysics_Body *body, double roll)
{
   if (!body)
     {
        ERR("Can't apply torque impulse to a null body.");
        return;
     }

   body->rigid_body->activate(1);
   body->rigid_body->applyTorqueImpulse(btVector3(0, 0, roll));
}

EAPI void
ephysics_body_rotation_on_z_axis_enable_set(EPhysics_Body *body, Eina_Bool enable)
{
   if (!body)
     {
        ERR("Can't set rotation on a null body.");
        return;
     }

   if (!enable)
     body->rigid_body->setAngularFactor(btVector3(0, 0, 0));
   else
     body->rigid_body->setAngularFactor(btVector3(0, 0, 1));
}

EAPI Eina_Bool
ephysics_body_rotation_on_z_axis_enable_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't check if rotation is enabled, body is null.");
        return EINA_FALSE;
     }

   return !!body->rigid_body->getAngularFactor().z();
}

EAPI double
ephysics_body_rotation_get(const EPhysics_Body *body)
{
   btTransform trans;
   double rot;

   if (!body)
     {
        ERR("Can't get rotation, body is null.");
        return 0;
     }

   body->rigid_body->getMotionState()->getWorldTransform(trans);
   rot = - trans.getRotation().getAngle() * RAD_TO_DEG *
      trans.getRotation().getAxis().getZ();

   return rot;
}

EAPI void
ephysics_body_data_set(EPhysics_Body *body, void *data)
{
   if (!body)
     {
        ERR("Can't set data, body is null.");
        return;
     }

   body->data = data;
}

EAPI void *
ephysics_body_data_get(const EPhysics_Body *body)
{
   if (!body)
     {
        ERR("Can't get data, body is null.");
        return NULL;
     }

   return body->data;
}

#ifdef  __cplusplus
}
#endif
